import unittest
import logging
from mock import patch, MagicMock

import soklog


class SoklogTest(unittest.TestCase):

    @patch('soklog.logging.getLogger')
    def test_init(self, mock):
        soklog.init(1, 2)

        mock.assert_called_with(2)
        self.assertEqual(1, soklog._DEFAULT.module)
        self.assertEqual(mock.return_value, soklog._DEFAULT.log)

    @patch('soklog.logging.getLogger')
    def test__get_args_as_string(self, mock):
        module = MagicMock()
        log = MagicMock()
        soklog_obj = soklog.SokLog(module, log)

        self.assertEqual('1 2 3', soklog_obj._get_args_as_string([1, 2, '3']))

    @patch.object(soklog._DEFAULT, 'log')
    def test_info(self, log_mock):
        soklog.info(1, 2, 3, something=4)
        log_mock.info.assert_called_with("1 2 3", something=4)

    @patch.object(soklog._DEFAULT, 'log')
    def test_warning(self, log_mock):
        soklog.warning(1, 2, 3, something=4)
        log_mock.warning.assert_called_with("1 2 3", something=4)

    @patch.object(soklog._DEFAULT, 'log')
    def test_debug(self, log_mock):
        with patch.object(soklog._DEFAULT, 'module', soklog):
            soklog.debug(1, 2, 3, something=4)
            log_mock.debug.assert_called_with(
                './tests.py:39 1 2 3', level=2, something=4)

    @patch.object(soklog._DEFAULT, 'log')
    def test_error(self, log_mock):
        soklog.error(1, 2, 3, something=4)
        log_mock.error.assert_called_with(1, 2, 3, something=4)

    @patch('soklog.logging.basicConfig')
    def test_start_stdout_logging(self, mock):
        soklog.start_stdout_logging()
        mock.assert_called_with(
            level=logging.DEBUG,
            format="%(asctime)-10s %(message)s",
            datefmt="%H:%M:%S"
        )

    @patch('soklog.logging')
    def test_start_file_logging(self, mock):
        with patch.object(soklog._DEFAULT, 'log') as log_mock:
            soklog.start_file_logging('somewhere')
            mock.FileHandler.assert_called_with('somewhere')
            mock.Formatter.assert_called_with("%(asctime)-10s %(message)s")

            hdlr = mock.FileHandler.return_value
            hdlr.setFormatter.assert_called_with(mock.Formatter.return_value)

            log_mock.addHandler.assert_called_with(hdlr)
            log_mock.setLevel.assert_called_with(mock.DEBUG)



import logging
import inspect
from os import path


class SokLog(object):

    def __init__(self, module, log_name):
        self.setUpModule(module, log_name)

    def _get_args_as_string(self, args):
        args = list(args)
        args = [str(arg) for arg in args]
        return ' '.join(args)

    def info(self, *args, **kwargs):
        msg = self._get_args_as_string(args)
        self.log.info(msg, **kwargs)

    def warning(self, *args, **kwargs):
        msg = self._get_args_as_string(args)
        self.log.warning(msg, **kwargs)

    def debug(self, *args, **kwargs):
        level = kwargs.get('level', 1)
        msg = self._get_args_as_string(args)
        main_path = path.dirname(self.module.__file__)
        src_path = inspect.stack()[level][1]
        src_path = '.' + src_path[len(main_path):]
        line = inspect.stack()[level][2]
        msg = '%s:%d %s' % (src_path, line, msg)
        self.log.debug(msg, **kwargs)

    def error(self, *args, **kwargs):
        self.log.error(*args, **kwargs)

    def start_file_logging(self, path):
        hdlr = logging.FileHandler(path)
        fmt = logging.Formatter("%(asctime)-10s %(message)s")
        hdlr.setFormatter(fmt)
        self.log.addHandler(hdlr)
        self.log.setLevel(logging.DEBUG)

    def start_stdout_logging(self):
        logging.basicConfig(
            level=logging.DEBUG,
            format="%(asctime)-10s %(message)s",
            datefmt="%H:%M:%S"
        )

    def setUpModule(self, module, log_name):
        self.module = module
        self.log = logging.getLogger(log_name)

_DEFAULT = SokLog(None, None)


def init(module, log_name):
    _DEFAULT.__init__(module, log_name)


def info(*args, **kwargs):
    return _DEFAULT.info(*args, **kwargs)


def warning(*args, **kwargs):
    return _DEFAULT.warning(*args, **kwargs)


def debug(*args, **kwargs):
    kwargs['level'] = 2
    return _DEFAULT.debug(*args, **kwargs)


def error(*args, **kwargs):
    return _DEFAULT.error(*args, **kwargs)


def start_file_logging(*args, **kwargs):
    return _DEFAULT.start_file_logging(*args, **kwargs)


def start_stdout_logging(*args, **kwargs):
    return _DEFAULT.start_stdout_logging(*args, **kwargs)



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages


install_requires = [
]

dependency_links = [
]

if __name__ == '__main__':
    setup(name='soklog',
          version='0.1',
          author=['Dominik "Socek" Długajczy'],
          author_email=['msocek@gmail.com', ],
          packages=find_packages(),
          install_requires=install_requires,
          dependency_links=dependency_links,
          test_suite='soklog.tests',
          )
